/**
 ** EstadoPartida.java
 **
 ** Copyright (C) 1992, 1999, 2007, 2011 Mariano Alvarez Fernandez
 ** [e-mail: malfer at telefonica.net]
 **
 ** This file is part of the Parchis4A program.
 **
 ** Parchis4A is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** Parchis4A is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with Parchis4A.  If not, see <http://www.gnu.org/licenses/>.
 **
 **/

package com.fgrim.parchis4a;

import java.util.Random;

import android.os.Bundle;

public class EstadoPartida {
	private static final Random RNG = new Random();
	
    // pawn colors
	public static final int AMAR = 0;
	public static final int AZUL = 1;
	public static final int ROJO = 2;
	public static final int VERD = 3;
	
	// player types
	public static final int PERSONA = 0;
	public static final int MAQUINA = 1;
	public static final int NOJUEGA = 2;

	//actions
	public static final int ACC_TIRADADO = 0;
	public static final int ACC_PARADADO = 1;
//	public static final int ACC_ELIGEFICHA = 2;
	public static final int ACC_MUEVE = 3;
//	public static final int ACC_CUENTA10 = 4;
//	public static final int ACC_CUENTA20 = 5;
//	public static final int ACC_ACASA = 6;
	public static final int ACC_PASA = 7;
	public static final int ACC_PARTIDATERMINADA = 8;
	
	public int tipo_jugador[];
	public boolean ha_salido[];
	public int ha_terminado[];
	public int sig_pos_fin;
	public Posicion pos_ficha[][];
	public int nfcasa[];
	public int nfmeta[];
	public int ultf_selec[];
	
	public int turno;
	public int nseises;
	public int nfmovida;

	public int accion;
	public int vdado;
	public GrupoJugadas gjg;
	public int njgelegida;
	public int sigvdado;
	
	public EstadoPartida() {
		tipo_jugador = new int[4];
		ha_salido = new boolean[4];
		ha_terminado = new int[4];
		pos_ficha =  new Posicion[4][4];
		nfcasa = new int[4];
		nfmeta = new int[4];
		ultf_selec = new int[4];
	}

	public EstadoPartida(EstadoPartida auxEP) {
		this();
		for (int c=AMAR; c<=VERD; c++) {
			tipo_jugador[c] = auxEP.tipo_jugador[c];
			ha_salido[c] = auxEP.ha_salido[c];
			ha_terminado[c] = auxEP.ha_terminado[c];
			for (int nf=0; nf<4; nf++) {
				pos_ficha[c][nf] = new Posicion();
				pos_ficha[c][nf].setPosicion(auxEP.pos_ficha[c][nf]);
			}
			nfcasa[c] = auxEP.nfcasa[c];
			nfmeta[c] = auxEP.nfmeta[c];
			ultf_selec[c] = auxEP.ultf_selec[c];
		}
		sig_pos_fin = auxEP.sig_pos_fin;
		turno = auxEP.turno;
		nseises = auxEP.nseises;
		nfmovida = auxEP.nfmovida;
	}
	
    private int[] pos_ficha_to_array() {
    	int pfa[] = new int[4*4*3];
    	int k = 0;
		for (int c=AMAR; c<=VERD; c++) {
			for (int nf=0; nf<4; nf++) {
				pfa[k++] = pos_ficha[c][nf].lugar;
				pfa[k++] = pos_ficha[c][nf].pos;
				pfa[k++] = pos_ficha[c][nf].ind;
			}
		}
    	return pfa;
    }
    
    private void array_to_pos_ficha(int pfa[]) {
    	int k = 0;
		for (int c=AMAR; c<=VERD; c++) {
			for (int nf=0; nf<4; nf++) {
				pos_ficha[c][nf] = new Posicion();
				pos_ficha[c][nf].lugar = pfa[k++];
				pos_ficha[c][nf].pos = pfa[k++];
				pos_ficha[c][nf].ind = pfa[k++];
			}
		}
    }
    
	/* Save game state */
    public Bundle saveState() {
        Bundle map = new Bundle();

        map.putIntArray("tj", tipo_jugador);
        map.putBooleanArray("hs", ha_salido);
        map.putIntArray("ht", ha_terminado);
        map.putInt("spfi", sig_pos_fin);
        map.putIntArray("pf", this.pos_ficha_to_array());
        map.putIntArray("nfcs", nfcasa);
        map.putIntArray("nfmt", nfmeta);
        map.putIntArray("ulfs", ultf_selec);
        map.putInt("tur", turno);
        map.putInt("n6", nseises);
        map.putInt("nfmv", nfmovida);
        map.putInt("acc", accion);
        map.putInt("vd", vdado);
        map.putInt("njge", njgelegida);
        map.putInt("sigv", sigvdado);

        return map;
    }

    public void restoreState(Bundle icicle) {
    	tipo_jugador = icicle.getIntArray("tj");
    	ha_salido = icicle.getBooleanArray("hs");
    	ha_terminado = icicle.getIntArray("ht");
    	sig_pos_fin = icicle.getInt("spfi");
    	this.array_to_pos_ficha(icicle.getIntArray("pf"));
    	nfcasa = icicle.getIntArray("nfcs");
    	nfmeta = icicle.getIntArray("nfmt");
    	ultf_selec = icicle.getIntArray("ulfs");
    	turno = icicle.getInt("tur");
    	nseises = icicle.getInt("n6");
    	nfmovida = icicle.getInt("nfmv");
    	accion = icicle.getInt("acc");
    	vdado = icicle.getInt("vd");
    	njgelegida = icicle.getInt("njge");
    	sigvdado = icicle.getInt("sigv");
   }
    
	public void iniciaPartida(int tjamar, int tjazul, int tjrojo, int tjverd) {
		tipo_jugador[AMAR] = tjamar;
		tipo_jugador[AZUL] = tjazul;
		tipo_jugador[ROJO] = tjrojo;
		tipo_jugador[VERD] = tjverd;
		for (int c=AMAR; c<=VERD; c++) {
			ha_salido[c] = false;
			ha_terminado[c] = -1;
			for (int nf=0; nf<4; nf++) {
				pos_ficha[c][nf] = new Posicion(Posicion.CASA, 0, nf);
			}
			nfcasa[c] = 4;
			nfmeta[c] = 0;
			ultf_selec[c] = -1;
		}
		
		sig_pos_fin = 0;
		turno = VERD; // so next player is AMAR!
        nseises = 0;
        
		accion = ACC_TIRADADO;
		sigvdado = 0;
        njgelegida = -1;
		this.avanzaTurno();
	}
	
    private void sumaTurno() {
		if (turno == VERD)
			turno = AMAR;
		else
			turno++;
    }
    
	public void avanzaTurno() {
		boolean encontrado = false;

		nseises = 0;
		nfmovida = -1;
		sumaTurno();
		
		for (int i=0; i<4; i++) {
			if (tipo_jugador[turno] != NOJUEGA && nfmeta[turno] < 4) {
				encontrado = true;
				break;
			} else
				sumaTurno();
		}
		
		if (!encontrado)
			accion = ACC_PARTIDATERMINADA;
	}
	
	public void iniciaAccion(Tablero mTbl) {
		if (accion == ACC_PARADADO) {
			vdado = RNG.nextInt(6) + 1;
//			vdado = RNG.nextInt(2) + 5;
			nseises = (vdado == 6) ? nseises+1 : 0;
			if (vdado == 6 && nfcasa[turno] == 0) vdado = 7; // hace 6 a 7
		} else if (accion == ACC_MUEVE) {
//			gjg.jg[njgelegida].impLog();
			sigvdado = mTbl.realizaJugada(gjg.jg[njgelegida]);
			if (gjg.njugadas > 1) {
				ultf_selec[gjg.jg[njgelegida].cjugador] = gjg.jg[njgelegida].nficha;
			}
            njgelegida = -1;
		} else if (accion == ACC_PASA) {
			sigvdado = 0;
		}
	}

	private int jugadaPorDefecto(int nj) {
		int jeleg = 0;
		
		if (nj < 2) return jeleg;
		int color = gjg.jg[0].cjugador;
		if (ultf_selec[color] < 0) return jeleg;
		for (int i=0; i<nj; i++) {
			if (gjg.jg[i].nficha == ultf_selec[color]) {
				jeleg = i;
				break;
			}
		}
		return jeleg;
	}
	
	public void terminaAccion(Tablero mTbl) {
		if (accion == ACC_TIRADADO) {
			accion = ACC_PARADADO;
		} else if (accion == ACC_PARADADO) {
            gjg = new GrupoJugadas();
            int n = gjg.genera(mTbl, turno, vdado);
            if (n > 0) {
                njgelegida = jugadaPorDefecto(n);
                if (tipo_jugador[turno] == MAQUINA)
                	eligeJugadaMaquina(mTbl);
            	accion = ACC_MUEVE;
            } else {
                njgelegida = -1;
            	accion = ACC_PASA;
            }
		} else if (accion == ACC_PASA) {
		    accion = ACC_TIRADADO;
		    if (sigvdado == -1 || nseises == 0 || nseises >2)
  		        avanzaTurno();
		} else if (accion == ACC_MUEVE) {
			if (sigvdado == 10 || sigvdado == 20) {
				vdado = sigvdado;
	            gjg = new GrupoJugadas();
	            int n = gjg.genera(mTbl, turno, vdado);
	            if (n > 0) {
	                njgelegida = jugadaPorDefecto(n);
	                if (tipo_jugador[turno] == MAQUINA)
	                	eligeJugadaMaquina(mTbl);
	            	accion = ACC_MUEVE;
	            } else {
	                njgelegida = -1;
	            	accion = ACC_PASA;
	            }
			} else {
  		        accion = ACC_TIRADADO;
  		        if (sigvdado == -1 || nseises == 0 || nseises >2)
		            avanzaTurno();
			}
	    }
	}
	
	public void eligeJugadaMaquina(Tablero mTbl) {
		int i, v, valor = -32000;
		
        if (gjg.njugadas < 2) return;
        
		gjg.completaInfoAnl(mTbl);
		for (i=0; i<gjg.njugadas; i++) {
			v = gjg.jg[i].calculaPeso();
			if (v > valor) {
				valor = v;
				njgelegida = i;
			}
		}
	}
	
	public boolean haComenzado() {
		for (int c=AMAR; c<=VERD; c++) {
	        if (nfcasa[c] < 4) return true;
		}
		return false;
	}
	
	public boolean haTerminado() {
		for (int c=AMAR; c<=VERD; c++) {
		    if (tipo_jugador[c] != NOJUEGA && nfmeta[c] < 4) return false;
		}
		return true;
	}
}
